package net.sf.aislib.goodies.rememberme;

import java.util.Date;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.acegisecurity.Authentication;
import org.acegisecurity.providers.rememberme.RememberMeAuthenticationToken;
import org.acegisecurity.ui.rememberme.TokenBasedRememberMeServices;
import org.acegisecurity.userdetails.UserDetails;
import org.acegisecurity.userdetails.UsernameNotFoundException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.StringUtils;

/**
 * Implementation of <code>RememberMeServices</code> using persistent storage on server side
 * and cookie on client side.
 *
 */
public class DaoRememberMeServices  extends TokenBasedRememberMeServices {

  private RememberMeDao dao;

  /**
   * @see org.acegisecurity.ui.rememberme.RememberMeServices#autoLogin(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
   */
  @Override
  public Authentication autoLogin(HttpServletRequest request, HttpServletResponse response) {
    Cookie[] cookies = request.getCookies();

    if ((cookies == null) || (cookies.length == 0)) {
      return null;
    }
    for (int i = 0; i < cookies.length; i++) {
      if (getCookieName().equals(cookies[i].getName())) {
        String cookieValue = cookies[i].getValue();
        if (Base64.isArrayByteBase64(cookieValue.getBytes())) {
          // Decode token from Base64
          // format of token is:
          //    tokenId + ":" +  expiryTime + ":" + Md5Hex(username + ":" + expiryTime + ":" + password + ":" + key)
          String cookieAsPlainText = new String(Base64.decodeBase64(cookieValue.getBytes()));
          String[] cookieTokens = StringUtils.delimitedListToStringArray(cookieAsPlainText, ":");

          if (cookieTokens.length == 3) {
            Long tokenId = null;
            // try to read token id first, if it will fail, cancel cookie, and return
            try {
              tokenId = new Long(cookieTokens[0]);
            } catch (NumberFormatException ex) {
              cancelCookie(request, response, "Cookie token[0] did not contain a valid number (contained '"
                + cookieTokens[0] + "')");
              return null;
            }

            // load Authentication Token from database
            AuthenticationToken authToken = dao.getAuthenticationToken(tokenId);
            if (authToken == null) {
              cancelCookie(request, response, "Authentication Token not found for '" + cookieTokens[0] + "')");
              return null;
            }

            // Check if username is the same
            if (!cookieTokens[1].equals(authToken.getUsername())) {
              dao.removeAuthenticationToken(tokenId);
              cancelCookie(request, response, "Authentication Token contatins incorrect username");
              return null;
            }

            // Check if it has not expired
            if (authToken.getExpiryTime().longValue() < System.currentTimeMillis()) {
              dao.removeAuthenticationToken(tokenId);
              cancelCookie(request, response, "Authentication token has expired (expired on '"
                + new Date(authToken.getExpiryTime().longValue()) + "'; current time is '" + new Date() + "')");
              return null;
            }

            // Check the user exists
            UserDetails userDetails = null;
            try {
              userDetails = getUserDetailsService().loadUserByUsername(authToken.getUsername());

            } catch (UsernameNotFoundException notFound) {
              dao.removeAuthenticationToken(tokenId);
              cancelCookie(request, response, "Authentication token contained username '" + authToken.getUsername()
                + "' but was not found");
              return null;
            }

            // Immediately reject if the user is not allowed to login
            if (!userDetails.isAccountNonExpired() || !userDetails.isCredentialsNonExpired()
              || !userDetails.isEnabled()) {
              cancelCookie(request, response, "Authentication token contained username '" + authToken.getUsername()
                + "' but account has expired, credentials have expired, or user is disabled");
              return null;
            }

            if (!authToken.getAuthToken().equals(cookieTokens[2])) {
              cancelCookie(request, response, "Cookie token[1] contained signature '" + cookieTokens[1]
                + "' but expected '" + authToken.getAuthToken() + "'");
              return null;
            }

            RememberMeAuthenticationToken auth = new RememberMeAuthenticationToken(getKey(), userDetails, userDetails
              .getAuthorities());
            auth.setDetails(authenticationDetailsSource.buildDetails(request));

            return auth;
          }
          cancelCookie(request, response, "Cookie token did not contain 3 tokens; decoded value was '"
            + cookieAsPlainText + "'");
          return null;
        }
        cancelCookie(request, response, "Cookie token was not Base64 encoded; value was '" + cookieValue + "'");
        return null;
      }
    }

    return null;
  }

  protected AuthenticationToken createToken(UserDetails userDetails, String password) {
    AuthenticationToken authToken = new AuthenticationToken();
    authToken.setUsername(userDetails.getUsername());
    // compute expiry time
    authToken.setExpiryTime(new Long(System.currentTimeMillis() + (getTokenValiditySeconds() * 1000)));
    // initialize token value
    authToken.setAuthToken(DigestUtils.md5Hex(authToken.getExpiryTime() + ":" + password + ":" + getKey()));
    return authToken;
  }

  /**
   * @see org.acegisecurity.ui.rememberme.RememberMeServices#loginSuccess(HttpServletRequest, HttpServletResponse, Authentication)
   */
  @Override
  public void loginSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication) {
    //  Exit if the principal hasn't asked to be remembered
    if (!rememberMeRequested(request, getParameter())) {
      return;
    }

    UserDetails userDetails = null;
    String password = null;

    if (authentication.getPrincipal() instanceof UserDetails) {
      userDetails = (UserDetails) authentication.getPrincipal();
      password = userDetails.getPassword();
    } else {
      // incorrect state, we do not have password
      return;
    }

    AuthenticationToken authToken = createToken(userDetails, password);

    // remember the token in database
    dao.persistAuthenticationToken(authToken);
    // and add the cookie
    response.addCookie(makeValidCookie(new String(Base64
      .encodeBase64(new String(authToken.getTokenId() + ":" + authToken.getUsername() + ":" + authToken.getAuthToken()).getBytes())), request, authToken.getExpiryTime().longValue()));
  }

  @Required
  public void setDao(RememberMeDao dao) {
    this.dao = dao;
  }
}