package net.sf.aislib.goodies.rememberme.jdbc;

import java.sql.Types;

import net.sf.aislib.goodies.rememberme.AuthenticationToken;
import net.sf.aislib.goodies.rememberme.RememberMeDao;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.PreparedStatementCreatorFactory;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.support.GeneratedKeyHolder;

/**
 * Implementation of {@link RememberMeDao} using JDBC calls.
 *
 * Structure for MySQL:
 * <pre>
 * CREATE TABLE rememeberme_tokens (
 *   token_id    INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
 *   token_value TEXT NOT NULL,
 *   username    VARCHAR( 40 ) NOT NULL,
 *   expiry_time BIGINT(20) NOT NULL
 * ) ENGINE = InnoDB;
 * </pre>
 */
public class JdbcRememberMeDao extends SimpleJdbcDaoSupport implements RememberMeDao {

  private String selectQuery = "select token_id, token_value, username, expiry_time from rememeberme_tokens where token_id = ?";
  private String deleteQuery = "delete from rememeberme_tokens where token_id = ?";
  private String insertQuery = "insert into rememeberme_tokens (token_value, username, expiry_time) values (?, ?, ?)";
  private String updateQuery = "update rememeberme_tokens set token_value = ?, username = ?, expiry_time = ? where token_id = ?";

  private ParameterizedRowMapper<AuthenticationToken> rowMapper = new AuthenticationTokenRowMapper();

  /**
   * @see RememberMeDao#getAuthenticationToken(Long)
   */
  public AuthenticationToken getAuthenticationToken(Long tokenId) {
    try {
      return getSimpleJdbcTemplate().queryForObject(selectQuery, rowMapper, tokenId);
    } catch (EmptyResultDataAccessException erdae) {
      return null;
    }
  }


  public Long persistAuthenticationToken(AuthenticationToken token) {
    if (token.getTokenId() == null) {
      PreparedStatementCreatorFactory psc = new PreparedStatementCreatorFactory(insertQuery);
      psc.addParameter(new SqlParameter("token_value", Types.VARCHAR));
      psc.addParameter(new SqlParameter("username", Types.VARCHAR));
      psc.addParameter(new SqlParameter("expiry_time", Types.NUMERIC));
      GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
      psc.setReturnGeneratedKeys(true);
      getJdbcTemplate().update(
        psc.newPreparedStatementCreator(
          new Object[] {token.getAuthToken(), token.getUsername(), token.getExpiryTime(),}), keyHolder);
      Long key = new Long(keyHolder.getKey().longValue());
      token.setTokenId(key);
      return key;
    } else {
      getSimpleJdbcTemplate().update(updateQuery, token.getAuthToken(), token.getUsername(), token.getExpiryTime(), token.getTokenId());
      return token.getTokenId();
    }
  }

  public void removeAuthenticationToken(Long tokenId) {
    getSimpleJdbcTemplate().update(deleteQuery, tokenId);
  }

  /**
   * Sets query for use in {@link #getAuthenticationToken(Long)}.
   *
   * @param selectQuery
   */
  public void setSelectQuery(String selectQuery) {
    this.selectQuery = selectQuery;
  }

  /**
   * Sets query for use in {@link #removeAuthenticationToken(Long).
   *
   * @param deleteQuery
   */
  public void setDeleteQuery(String deleteQuery) {
    this.deleteQuery = deleteQuery;
  }

  /**
   * Sets query for inserting new record, for use in {@link #persistAuthenticationToken(AuthenticationToken)}.
   *
   * @param insertQuery
   */
  public void setInsertQuery(String insertQuery) {
    this.insertQuery = insertQuery;
  }

  /**
   * Sets query for updating record, for use in {@link #persistAuthenticationToken(AuthenticationToken)}.
   *
   * @param insertQuery
   */
  public void setUpdateQuery(String updateQuery) {
    this.updateQuery = updateQuery;
  }

  /**
   * Sets row mapper for reading tokens.
   *
   * @param rowMapper
   */
  public void setRowMapper(ParameterizedRowMapper<AuthenticationToken> rowMapper) {
    this.rowMapper = rowMapper;
  }
}
